<?php

namespace Drupal\migrate_sandbox\Form;

use Drupal\Component\Serialization\Yaml;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\migrate\MigrateExecutable;
use Drupal\migrate\Plugin\MigrationInterface;
use Drupal\migrate_sandbox\MigrateSandboxExecutable;
use Drupal\migrate_sandbox\MigrateSandboxMessage;
use Drupal\migrate_sandbox\SandboxMigration;

/**
 * Configure Migrate Sandbox settings for this site.
 */
class MigrateSandboxForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'migrate_sandbox_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['migrate_sandbox.settings', 'migrate_sandbox.processed'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildForm($form, $form_state);
    $form['intro'] = [
      '#markup' => '<p>Play around with the process pipeline using this form. Nothing gets saved to your db. Errors get shown as messages instead of hiding in logs. It is fast and fun.</p><h3>Limitations</h3><ul><li>You only get one row of embedded data.</li><li>migrate_lookup cannot be used</li></ul>'
    ];
    $form['source_wrapper'] = [
      '#type' => 'fieldset',
      '#title' => 'source',
      '#tree' => FALSE,
    ];
    $form['source_wrapper']['data_rows'] = [
      '#type' => 'textarea',
      '#title' => 'data_rows:',
      '#description' => $this->t('You can only enter one row. It can have lots of properties. But just one row.'),
      '#attributes' => ['data-yaml-editor' => 'true'],
      '#default_value' => Yaml::encode($this->config('migrate_sandbox.settings')->get('source.data_rows')[0]),
    ];
    $form['source_wrapper']['constants'] = [
      '#type' => 'textarea',
      '#title' => 'constants:',
      '#attributes' => ['data-yaml-editor' => 'true'],
      '#default_value' => Yaml::encode($this->config('migrate_sandbox.settings')->get('source.constants')),
    ];
    $form['source_wrapper']['ids'] = [
      '#type' => 'textarea',
      '#title' => $this->t('ids:'),
      '#attributes' => ['data-yaml-editor' => 'true'],
      '#default_value' => Yaml::encode($this->config('migrate_sandbox.settings')->get('source.ids')),
    ];
    $form['process_wrapper'] = [
      '#type' => 'fieldset',
      '#title' => 'process',
      '#tree' => FALSE,
    ];
    $form['process_wrapper']['process'] = [
      '#type' => 'textarea',
      '#title' => 'process:',
      '#attributes' => ['data-yaml-editor' => 'true'],
      '#default_value' => Yaml::encode($this->config('migrate_sandbox.settings')->get('process')),
    ];
    $form['results'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Results'),
      '#attributes' => ['data-yaml-editor' => 'true', 'readonly' => 'readonly'],
      '#default_value' => Yaml::encode($this->config('migrate_sandbox.processed')->get()),
      '#weight' => 1000,
    ];
    $form['actions']['submit']['#value'] = $this->t('Save & Run');
    $form['outro'] = [
      '#markup' => '<p>After submitting, <code>migrate_sandbox.settings</code> and <code>migrate_sandbox.processed</code> will be updated along with the results below.</p>'
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $yaml_fields = [
      'data_rows',
      'constants',
      'ids',
      'process',
    ];
    foreach ($yaml_fields as $field) {
      try {
        Yaml::decode($form_state->getValue($field));
      }
      catch (\Exception $e) {
        $form_state->setErrorByName($field, $e->getMessage());
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $id = 'migrate_sandbox';
    $label = 'Migrate Sandbox';
    $source = [
      'plugin' => 'embedded_data',
      'data_rows' => [Yaml::decode($form_state->getValue('data_rows'))],
      'constants' => Yaml::decode($form_state->getValue('constants')),
      'ids' => Yaml::decode($form_state->getValue('ids')),
    ];
    $process = Yaml::decode($form_state->getValue('process'));
    $destination = [
      'plugin' => 'config',
      'config_name' => 'migrate_sandbox.processed',
    ];
    $this->config('migrate_sandbox.settings')
      ->setData([
        'id' => $id,
        'label' => $label,
        'source' => $source,
        'process' => $process,
        'destination' => $destination,
      ])
      ->save();
    $this->runMigration();
  }

  /**
   * {@inheritdoc}
   */
  protected function runMigration() {
    $definition = $this->config('migrate_sandbox.settings')->getRawData();
    $migration = SandboxMigration::create(\Drupal::getContainer(), [], 'migrate_sandbox', $definition);
    $migration->setStatus(MigrationInterface::STATUS_IDLE);
    $executable = new MigrateExecutable($migration, new MigrateSandboxMessage());
    try {
      $this->config('migrate_sandbox.processed')->setData([])->save();
      $executable->import();
      return $this->config('migrate_sandbox.processed')->get();
    }
    catch (\Throwable $e) {
      \Drupal::messenger()->addError($e->getMessage());
      return [];
    }
  }

}
